# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
import datetime
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString as HL7String,
    HprimDate,
    HprimList as HL7List,
    HprimForeign as HL7Foreign,
)

logger = logging.getLogger("driverng.starlet")


class HL7Date(HprimDate):

    def to_hprim(self, value):
        return value.strftime("%Y%m%d%H%M%S")

    def to_python(self, value):
        return datetime.datetime.strptime(value.split("+")[0], "%Y%m%d%H%M%S.%f")


class StarletSegmentMSH(HeaderSegment):
    sending_application = HL7String(3, default="MCA")
    sending_facility = HL7String(4, default="MCA")
    receiving_application = HL7String(5, default="Seegene Viewer")
    receiving_facility = HL7String(6, default="Seegene")
    date = HL7Date(7)
    message_type = HL7String((9, 1), default="ORU")
    message_code = HL7String((9, 2), default="R30")
    message_version = HL7String((9, 3))
    version = HL7String(12, default="2.5")

    patients = HL7Foreign("PID", "header")
    orders = HL7Foreign("OBR", "header")


class StarletSegmentPID(Segment):
    external_patient_id = HL7String(2)
    internal_patient_id = HL7String(3)
    alternate_patient_id = HL7String(4)
    first_name = HL7String((6, 1))
    last_name = HL7String((6, 2))

    orders = HL7Foreign("OBR", "patient")
    info_action = HL7Foreign("ORC", "patient")
    infos = HL7Foreign("NTE", "patient")

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(StarletSegmentPID, self).to_string(separators=separators)


class StarletSegmentORC(Segment):
    order_control = HL7String(1)

    orders = HL7Foreign("OBR", "patient")

    def to_string(self, separators=None):
        self.rang_segment = "NW"
        return super(StarletSegmentORC, self).to_string(separators=separators)


class StarletSegmentOBR(Segment):
    well_name = HL7String((4, 1))
    well_id = HL7String((4, 2))
    product_name = HL7String((5, 1))
    auto_interpretation = HL7String((5, 2))
    comment = HL7String((5, 3))

    common_order = HL7Foreign("ORC", "order")
    infos = HL7Foreign("NTE", "order")

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(StarletSegmentOBR, self).to_string(separators=separators)


class StarletSegmentNTE(Segment):
    comment = HL7String(3)
    # dye~target~decision~result_type~ct~result_value
    result = HL7List(4)

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(StarletSegmentNTE, self).to_string(separators=separators)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "starlet",
        "label": "Starlet CFX-96 Seegene HL7",
    }
    SEGMENT_CLASS_MAP = {
        "MSH": StarletSegmentMSH,
        "PID": StarletSegmentPID,
        "ORC": StarletSegmentORC,
        "OBR": StarletSegmentOBR,
        "NTE": StarletSegmentNTE,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def create(self, **kwargs):
        return StarletSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        msg = self.create(
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
                unique_together=["action"],
                action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            for sample, action in patient.samples:
                seg_pid = msg.patients.add(
                    internal_patient_id=sample.sample_id,
                    last_name=patient.last_name,
                    first_name=patient.first_name,
                )
                for order in sample["orders"]:
                    seg_orc = seg_pid.info_action.add()
                    seg_obr = seg_orc.orders.add()
                    seg_obr.product_name = sample.sample_id
                    order_code = [order.code]
                    order_code.append(sample.specimen)
                    seg_obr.infos.add(result=order_code)
        if msg.patients:
            return msg
        return msg

    def decode_message(self, message):
        if message.message_type != "ORU":
            raise NotImplementedError

        worklist = Worklist(context="result")
        for seg_obr in message.orders:
            if "Control" in seg_obr.auto_interpretation:
                sample = worklist.get_or_create(
                    sample_id=seg_obr.well_name,
                    control_lot=seg_obr.well_name,
                    type="control"
                )
            else:
                sample = worklist.get_or_create(
                    sample_id=seg_obr.well_name,
                    rack=seg_obr.well_id,
                    type="patient"
                )
            for seg_nte in seg_obr.infos:
                if len(seg_nte.result) != 6:
                    continue
                if seg_nte.result[1] and seg_nte.result[3]:
                    sample.results.add(
                        code=seg_nte.result[1],
                        value=seg_nte.result[3],
                        date=message.date
                    )
                # LEVEL RESULT
                if seg_nte.result[1] and seg_nte.result[5] and seg_nte.result[5] != "N/A":
                    sample.results.add(
                        code="LEVEL_{}".format(seg_nte.result[1]),
                        value=seg_nte.result[5],
                        date=message.date
                    )
        return worklist
